@@ -43,7 +43,6 @@
    ekr@rtfm.com  Tue Dec 29 15:13:03 1998
  */
 
-static char *RCSSTRING="$Id: tcpconn.c,v 1.7 2002/08/17 01:33:16 ekr Exp $";
 
 #include "network.h"
 #include "tcpconn.h"
@@ -71,13 +70,8 @@ static int zero_conn(conn)
     return(0);
   }
 
-int tcp_find_conn(connp,directionp,saddr,sport,daddr,dport)
-  tcp_conn **connp;
-  int *directionp;
-  struct in_addr *saddr;
-  u_short sport;
-  struct in_addr *daddr;
-  u_short dport;
+int tcp_find_conn(tcp_conn **connp, int *directionp,struct in_addr *saddr,
+	u_short sport, struct in_addr *daddr, u_short dport)
   {
     conn_struct *conn;
 
@@ -107,12 +101,8 @@ int tcp_find_conn(connp,directionp,saddr,sport,daddr,dport)
     return(R_NOT_FOUND);
   }
 
-int tcp_create_conn(connp,i_addr,i_port,r_addr,r_port)
-  tcp_conn **connp;
-  struct in_addr *i_addr;
-  u_short i_port;
-  struct in_addr *r_addr;
-  u_short r_port;
+int tcp_create_conn(tcp_conn **connp,struct in_addr *i_addr,
+	u_short i_port, struct in_addr *r_addr, u_short r_port)
   {
     conn_struct *conn=0;
 
@@ -180,7 +170,8 @@ int clean_old_conn() {
     while(conn) {
         tcpconn = &conn->conn;
         conn=conn->next;
-        timestamp_diff(&last_packet_seen_time, &tcpconn->last_seen_time, &dt);
+        if(timestamp_diff(&last_packet_seen_time, &tcpconn->last_seen_time, &dt))
+            continue;
         if(dt.tv_sec > conn_ttl) {
             i++;
             tcp_destroy_conn(tcpconn);
@@ -190,8 +181,7 @@ int clean_old_conn() {
 }
 
 int destroy_all_conn() {
-    conn_struct *conn;
-    int i = 0,r;
+    int i = 0;
     while(first_conn) {
         i++;
         tcp_destroy_conn(&first_conn->conn);
@@ -226,7 +216,7 @@ int copy_tcp_segment_queue(out,in)
 	ABORT(R_NO_MEMORY);
       if(!base) base=*out;
 
-      if(r=packet_copy(in->p,&(*out)->p))
+      if((r=packet_copy(in->p,&(*out)->p)))
 	ABORT(r);
       out=&(*out)->next;  /* Move the pointer we're assigning to */
     }
