@@ -1,3 +1,4 @@
+#include <json-c/json.h>
 #include "network.h"
 #include "ssl_h.h"
 #include "sslprint.h"
@@ -15,6 +16,10 @@ static int decode_ContentType_ChangeCipherSpec(ssl,dir,seg,data)
   Data *data;
   {
 
+    struct json_object *jobj;
+
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "msg_type", json_object_new_string("ChangeCipherSpec"));
 
     ssl_process_change_cipher_spec(ssl,ssl->decoder,dir);
 
@@ -25,7 +30,7 @@ static int decode_ContentType_ChangeCipherSpec(ssl,dir,seg,data)
       ssl->r_state=SSL_ST_SENT_CHANGE_CIPHER_SPEC;
     }
 
-    printf("\n");
+    LF;
     return(0);
 
   }
@@ -37,9 +42,10 @@ static int decode_ContentType_Alert(ssl,dir,seg,data)
   {
 
    int r;
+   struct json_object *jobj;
 
    if(ssl->record_encryption==REC_CIPHERTEXT){
-     printf("\n");
+     LF;
      return(0);
    }
 
@@ -49,17 +55,20 @@ static int decode_ContentType_Alert(ssl,dir,seg,data)
 	ERETURN(R_EOD);
    }
 
+   jobj = ssl->cur_json_st;
+   json_object_object_add(jobj, "msg_type", json_object_new_string("Alert"));
+
    P_(P_HL){
-      printf("\n");
+      LF;
       SSL_DECODE_ENUM(ssl,"level",1,AlertLevel_decoder,P_HL,data,0);
-      printf("\n");
+      LF;
       SSL_DECODE_ENUM(ssl,"value",1,AlertDescription_decoder,P_HL,data,0);
-      printf("\n");
+      LF;
    }
    else {
          SSL_DECODE_ENUM(ssl,0,1,AlertLevel_decoder,SSL_PRINT_ALL,data,0);
          SSL_DECODE_ENUM(ssl,0,1,AlertDescription_decoder,SSL_PRINT_ALL,data,0);
-	 printf("\n");
+	 LF;
    }
    return(0);
 
@@ -77,9 +86,10 @@ static int decode_ContentType_Handshake(ssl,dir,seg,data)
     UINT4 t,l;
     int rs=0;
     Data d;
+    struct json_object *jobj;
 
     if(ssl->record_encryption==REC_CIPHERTEXT){
-      printf("\n");
+      LF;
       return(0);
     }
 
@@ -93,13 +103,16 @@ static int decode_ContentType_Handshake(ssl,dir,seg,data)
         ERETURN(R_EOD);
       }
 
+      jobj = ssl->cur_json_st;
+      json_object_object_add(jobj, "msg_type", json_object_new_string("Handshake"));
+
       d.data=data->data;
       d.len=l;
       data->len-=l;
       data->data+=l;
       P_(P_HL){
 	if(!rs){
-	  printf("\n");
+	  LF;
 	  rs=1;
 	}
       }
@@ -118,6 +131,9 @@ static int decode_ContentType_application_data(ssl,dir,seg,data)
 
     int r;
     Data d;
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "msg_type", json_object_new_string("application data"));
 
     SSL_DECODE_OPAQUE_ARRAY(ssl,"data",data->len,0,data,&d);
 
@@ -125,7 +141,7 @@ static int decode_ContentType_application_data(ssl,dir,seg,data)
 	    print_data(ssl,&d);
     }
     else {
-	printf("\n");
+	LF;
     }
     return(0);
 
@@ -160,9 +176,12 @@ static int decode_HandshakeType_HelloRequest(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("HelloRequest"));
 
 
-  printf("\n");
+  LF;
   return(0);
 
   }
@@ -173,6 +192,9 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
   Data *data;
   {
 
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("ClientHello"));
 
     UINT4 vj,vn,cs,cslen,complen,comp,odd,exlen,ex;
     Data session_id,random;
@@ -182,14 +204,14 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
     extern decoder compression_method_decoder[];
     extern decoder extension_decoder[];
 
-    printf("\n");
+    LF;
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT8(ssl,0,0,data,&vj);
     SSL_DECODE_UINT8(ssl,0,0,data,&vn);
 
     P_(P_HL) {explain(ssl,"Version %d.%d ",vj,vn);
-        printf("\n");
-   }
+        LF;
+    }
 
     SSL_DECODE_OPAQUE_ARRAY(ssl,"random",32,P_ND,data,&random);
     ssl_set_client_random(ssl->decoder,random.data,random.len);
@@ -216,10 +238,11 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
         }
 
 	    for(;cslen;cslen-=2){
-	      ssl_decode_enum(ssl,0,2,cipher_suite_decoder,
-	        0,data,&cs);
+	      if(ssl_decode_enum(ssl,0,2,cipher_suite_decoder,
+	        0,data,&cs))
+                return(1);
 	      ssl_print_cipher_suite(ssl,(vj<<8)|vn,P_HL,cs);
-	      printf("\n");
+	      LF;
 	    }
     }
 
@@ -228,7 +251,7 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
       explain(ssl,"compression methods\n");
       for(;complen;complen--){
         SSL_DECODE_ENUM(ssl,0,1,compression_method_decoder,P_HL,data,&comp);
-        printf("\n");
+        LF;
       }
     }
 
@@ -244,7 +267,7 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
     	  }
     	  continue;
     	}
-    	printf("\n");
+        LF;
       }
     }
     return(0);
@@ -263,14 +286,18 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
 
     extern decoder extension_decoder[];
 
-    printf("\n");
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("ServerHello"));
+
+    LF;
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT8(ssl,0,0,data,&vj);
     SSL_DECODE_UINT8(ssl,0,0,data,&vn);
 
     ssl->version=vj*256+vn;
     P_(P_HL) {explain(ssl,"Version %d.%d ",vj,vn);
-        printf("\n");
+        LF;
    }
 
 
@@ -288,9 +315,9 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
     ssl_process_server_session_id(ssl,ssl->decoder,session_id.data,
       session_id.len);
 
-    P_(P_HL) printf("\n");
+    P_(P_HL) LF;
     SSL_DECODE_ENUM(ssl,"compressionMethod",1,compression_method_decoder,P_HL,data,0);
-    P_(P_HL) printf("\n");
+    P_(P_HL) LF;
 
     SSL_DECODE_UINT16(ssl,"extensions len",0,data,&exlen);
     if (exlen) {
@@ -304,7 +331,7 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
     	  }
     	  continue;
     	}
-    	printf("\n");
+        LF;
       }
     }
 
@@ -323,10 +350,16 @@ static int decode_HandshakeType_Certificate(ssl,dir,seg,data)
     Data cert;
     int r;
 
-    printf("\n");
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("Certificate"));
+
+    LF;
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT24(ssl,"certificates len",0,data,&len);
 
+    json_object_object_add(jobj, "cert_chain", json_object_new_array());
+
     while(len){
       SSL_DECODE_OPAQUE_ARRAY(ssl,"certificate",-((1<<23)-1),
         0,data,&cert);
@@ -347,7 +380,11 @@ static int decode_HandshakeType_ServerKeyExchange(ssl,dir,seg,data)
 
    int r;
 
-    printf("\n");
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("ServerKeyExchange"));
+
+    LF;
     ssl_update_handshake_messages(ssl,data);
    if(ssl->cs){
      P_(P_ND){
@@ -385,14 +422,18 @@ static int decode_HandshakeType_CertificateRequest(ssl,dir,seg,data)
     Data ca;
     int r;
 
-    printf("\n");
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("CertificateRequest"));
+
+    LF;
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT8(ssl,"certificate_types len",0,data,&len);
     for(;len;len--){
       SSL_DECODE_ENUM(ssl,"certificate_types",1,
         client_certificate_type_decoder, P_HL,data,0);
       P_(P_HL){
-	printf("\n");
+	LF;
       }
     };
 
@@ -416,8 +457,11 @@ static int decode_HandshakeType_ServerHelloDone(ssl,dir,seg,data)
   Data *data;
   {
 
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("ServerHelloDone"));
 
-  printf("\n");
+  LF;
   ssl_update_handshake_messages(ssl,data);
   return(0);
 
@@ -431,9 +475,14 @@ static int decode_HandshakeType_CertificateVerify(ssl,dir,seg,data)
 
 
   int r;
-  printf("\n");
+
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("CertificateVerify"));
+
+  LF;
   ssl_update_handshake_messages(ssl,data);
-  SSL_DECODE_OPAQUE_ARRAY(ssl,"Signature",-(1<<15-1),P_HL,data,0);
+  SSL_DECODE_OPAQUE_ARRAY(ssl,"Signature",-((1<<15)-1),P_HL,data,0);
   return(0);
 
   }
@@ -448,14 +497,18 @@ static int decode_HandshakeType_ClientKeyExchange(ssl,dir,seg,data)
    int r;
    Data pms;
 
-    printf("\n");
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("ClientKeyExchange"));
+
+    LF;
     ssl_update_handshake_messages(ssl,data);
    if(ssl->cs){
      switch(ssl->cs->kex){
 
 	case KEX_RSA:
 	   if(ssl->version > 768) {
-	           SSL_DECODE_OPAQUE_ARRAY(ssl,"EncryptedPreMasterSecret",-(1<<15-1),
+	           SSL_DECODE_OPAQUE_ARRAY(ssl,"EncryptedPreMasterSecret",-((1<<15)-1),
 	             P_ND,data,&pms);
 
 	        }
@@ -486,7 +539,11 @@ static int decode_HandshakeType_Finished(ssl,dir,seg,data)
 
    int r;
 
-    printf("\n");
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "handshake_type", json_object_new_string("Finished"));
+
+    LF;
    switch(ssl->version){
      case 0x300:
        SSL_DECODE_OPAQUE_ARRAY(ssl,"md5_hash",16,P_ND,data,0);
@@ -496,7 +553,7 @@ static int decode_HandshakeType_Finished(ssl,dir,seg,data)
      case 0x301:
        SSL_DECODE_OPAQUE_ARRAY(ssl,"verify_data",12,P_ND,data,0);
 	P_(P_ND)
-	  printf("\n");
+	  LF;
        break;
    }
 
@@ -2124,6 +2181,9 @@ static int decode_AlertLevel_warning(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "alert_level", json_object_new_string("warning"));
 	return(0);
   }
 static int decode_AlertLevel_fatal(ssl,dir,seg,data)
@@ -2132,6 +2192,9 @@ static int decode_AlertLevel_fatal(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
+    struct json_object *jobj;
+    jobj = ssl->cur_json_st;
+    json_object_object_add(jobj, "alert_level", json_object_new_string("fatal"));
 	return(0);
   }
 decoder AlertLevel_decoder[]={
@@ -2521,8 +2584,8 @@ static int decode_extension_server_name(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
-    UINT4 t;
-    int l,r,p;
+    UINT4 t,l;
+    int r,p;
 
     extern decoder server_name_type_decoder[];
 
@@ -2530,7 +2593,7 @@ static int decode_extension_server_name(ssl,dir,seg,data)
 
     if(dir==DIR_I2R){
       SSL_DECODE_UINT16(ssl,"server name list length",0,data,&l);
-      printf("\n");
+      LF;
       while(l) {
 	p=data->len;
 	SSL_DECODE_UINT8(ssl, "server name type", 0, data, &t);
@@ -2557,7 +2620,8 @@ static int decode_extension_encrypt_then_mac(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
-    int l,r,*etm;
+    int r,*etm;
+    UINT4 l;
 
     etm=&ssl->extensions->encrypt_then_mac;
 
@@ -2574,7 +2638,8 @@ static int decode_extension_extended_master_secret(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
-    int l,r,*ems;
+    int r,*ems;
+    UINT4 l;
 
     ems=&ssl->extensions->extended_master_secret;
 
@@ -2591,7 +2656,8 @@ static int decode_extension(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
-    int l,r;
+    int r;
+    UINT4 l;
     SSL_DECODE_UINT16(ssl,"extension length",0,data,&l);
     data->len-=l;
     data->data+=l;
@@ -2630,16 +2696,86 @@ decoder extension_decoder[] = {
 		"status_request",
 		decode_extension
 	},
+        {
+                6,
+                "user_mapping",
+                decode_extension
+        },
+        {
+                7,
+                "client_authz",
+                decode_extension
+        },
+        {
+                8,
+                "server_authz",
+                decode_extension
+        },
+        {
+                9,
+                "cert_type",
+                decode_extension
+        },
+        {
+                10,
+                "supported_groups",
+                decode_extension
+        },
+        {
+                11,
+                "ec_point_formats",
+                decode_extension
+        },
+        {
+                12,
+                "srp",
+                decode_extension
+        },
 	{
 		13,
 		"signature_algorithms",
 		decode_extension
 	},
+        {
+                14,
+                "use_srtp",
+                decode_extension
+        },
+        {
+                15,
+                "heartbeat",
+                decode_extension
+        },
 	{
 		16,
 		"application_layer_protocol_negotiation",
 		decode_extension
 	},
+        {
+                17,
+                "status_request_v2",
+                decode_extension
+        },
+        {
+                18,
+                "signed_certificate_timestamp",
+                decode_extension
+        },
+        {
+                19,
+                "client_certificate_type",
+                decode_extension
+        },
+        {
+                20,
+                "server_certificate_type",
+                decode_extension
+        },
+        {
+                21,
+                "padding",
+                decode_extension
+        },
 	{
 		22,
 		"encrypt_then_mac",
@@ -2650,6 +2786,156 @@ decoder extension_decoder[] = {
 		"extended_master_secret",
 		decode_extension_extended_master_secret
 	},
+        {
+                24,
+                "token_binding",
+                decode_extension
+        },
+        {
+                25,
+                "cached_info",
+                decode_extension
+        },
+        {
+                26,
+                "tls_lts",
+                decode_extension
+        },
+        {
+                27,
+                "compress_certificate",
+                decode_extension
+        },
+        {
+                28,
+                "record_size_limit",
+                decode_extension
+        },
+        {
+                29,
+                "pwd_protect",
+                decode_extension
+        },
+        {
+                30,
+                "pwd_clear",
+                decode_extension
+        },
+        {
+                31,
+                "password_salt",
+                decode_extension
+        },
+        {
+                32,
+                "ticket_pinning",
+                decode_extension
+        },
+        {
+                33,
+                "tls_cert_with_extern_psk",
+                decode_extension
+        },
+        {
+                34,
+                "delegated_credentials",
+                decode_extension
+        },
+        {
+                35,
+                "session_ticket",
+                decode_extension
+        },
+        {
+                36,
+                "TLMSP",
+                decode_extension
+        },
+        {
+                37,
+                "TLMSP_proxying",
+                decode_extension
+        },
+        {
+                38,
+                "TLMSP_delegate",
+                decode_extension
+        },
+        {
+                39,
+                "supported_ekt_ciphers",
+                decode_extension
+        },
+        {
+                41,
+                "pre_shared_key",
+                decode_extension
+        },
+        {
+                42,
+                "early_data",
+                decode_extension
+        },
+        {
+                43,
+                "supported_versions",
+                decode_extension
+        },
+        {
+                44,
+                "cookie",
+                decode_extension
+        },
+        {
+                45,
+                "psk_key_exchange_modes",
+                decode_extension
+        },
+        {
+                47,
+                "certificate_authorities",
+                decode_extension
+        },
+        {
+                48,
+                "oid_filters",
+                decode_extension
+        },
+        {
+                49,
+                "post_handshake_auth",
+                decode_extension
+        },
+        {
+                50,
+                "signature_algorithms_cert",
+                decode_extension
+        },
+        {
+                51,
+                "key_share",
+                decode_extension
+        },
+        {
+                52,
+                "transparency_info",
+                decode_extension
+        },
+        {
+                53,
+                "connection_id",
+                decode_extension
+        },
+        {
+                55,
+                "external_id_hash",
+                decode_extension
+        },
+        {
+                56,
+                "external_session_id",
+                decode_extension
+        },
 	{
 		13172,
 		"next_protocol_negotiation",
@@ -2670,9 +2956,11 @@ static int decode_server_name_type_host_name(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
-    int l,r;
+    int r;
+    UINT4 l;
     SSL_DECODE_UINT16(ssl,"server name length",0,data,&l);
-    printf(": %.*s",l,data->data);
+    if(!(NET_print_flags & NET_PRINT_JSON))
+	printf(": %.*s",l,data->data);
 
     /* Possibly use data->data to set/modify ssl->server_name */
 	if (l!=0)
@@ -2682,6 +2970,7 @@ static int decode_server_name_type_host_name(ssl,dir,seg,data)
 		if (server_name != NULL)
 		{
 			if (ssl->server_name) free(ssl->server_name);
+			if (l > data->len) l = data->len;
 			memcpy(server_name,data->data,l);
 			ssl->server_name = server_name;
 		}
@@ -2697,7 +2986,8 @@ static int decode_server_name(ssl,dir,seg,data)
   segment *seg;
   Data *data;
   {
-    int l,r;
+    int r;
+    UINT4 l;
     SSL_DECODE_UINT16(ssl,"server name length",0,data,&l);
     data->len-=l;
     data->data+=l;
