@@ -44,8 +44,8 @@
  */
 
 
-static char *RCSSTRING="$Id: ssl_analyze.c,v 1.8 2002/01/21 18:46:13 ekr Exp $";
-
+#include <json-c/json.h>
+#include <arpa/inet.h>
 #include "network.h"
 #include "debug.h"
 #include "sslprint.h"
@@ -61,6 +61,7 @@ static int create_ssl_analyzer PROTO_LIST((void *handle,
   proto_ctx *ctx,tcp_conn *conn,proto_obj **objp,
   struct in_addr *i_addr,u_short i_port,
   struct in_addr *r_addr,u_short r_port, struct timeval *base_time));
+static int destroy_ssl_ctx PROTO_LIST((void *handle,proto_ctx **ctxp));
 static int destroy_ssl_analyzer PROTO_LIST((proto_obj **objp));
 static int read_ssl_record PROTO_LIST((ssl_obj *obj,r_queue *q,segment *seg,
   int offset,segment **lastp,int *offsetp));
@@ -188,7 +189,7 @@ static int parse_ssl_flags(str)
     
     y=str;
     
-    while(x=strtok(y,",")){
+    while((x=strtok(y,","))){
       y=0;
       
       if(*x=='!'){
@@ -219,7 +220,7 @@ static int create_ssl_ctx(handle,ctxp)
     ssl_decode_ctx *ctx=0;
     int r,_status;
     
-    if(r=ssl_decode_ctx_create(&ctx,SSL_keyfile,SSL_password,SSL_keylogfile))
+    if((r=ssl_decode_ctx_create(&ctx,SSL_keyfile,SSL_password,SSL_keylogfile)))
       ABORT(r);
 
     *ctxp=(proto_ctx *)ctx;
@@ -228,16 +229,19 @@ static int create_ssl_ctx(handle,ctxp)
     return(_status);
   }
 
-static int create_ssl_analyzer(handle,ctx,conn,objp,i_addr,i_port,r_addr,r_port,base_time)
+static int destroy_ssl_ctx(handle,ctxp)
   void *handle;
-  proto_ctx *ctx;
-  tcp_conn *conn;
-  proto_obj **objp;
-  struct in_addr *i_addr;
-  u_short i_port;
-  struct in_addr *r_addr;
-  u_short r_port;
-  struct timeval *base_time;
+  proto_ctx **ctxp;
+  {
+        ssl_decode_ctx *ctx=0;
+        ctx=(ssl_decode_ctx *) *ctxp;
+	ssl_decode_ctx_destroy(&ctx);
+	return 0;
+  }
+
+static int create_ssl_analyzer(void *handle, proto_ctx *ctx, tcp_conn *conn,
+  proto_obj **objp, struct in_addr *i_addr, u_short i_port, struct in_addr *r_addr,
+  u_short r_port, struct timeval *base_time)
   {
     int r,_status;
     ssl_obj *obj=0;
@@ -248,14 +252,20 @@ static int create_ssl_analyzer(handle,ctx,conn,objp,i_addr,i_port,r_addr,r_port,
     obj->ssl_ctx=(ssl_decode_ctx *)ctx;
     obj->conn=conn;
     
-    if(r=create_r_queue(&obj->r2i_queue))
+    if((r=create_r_queue(&obj->r2i_queue)))
       ABORT(r);
-    if(r=create_r_queue(&obj->i2r_queue))
+    if((r=create_r_queue(&obj->i2r_queue)))
       ABORT(r);
 
     lookuphostname(i_addr,&obj->client_name);
+    if(!(obj->client_ip=(char *)calloc(1,INET_ADDRSTRLEN)))
+      ABORT(R_NO_MEMORY);
+    inet_ntop(AF_INET, i_addr, obj->client_ip, INET_ADDRSTRLEN);
     obj->client_port=i_port;
-    lookuphostname(r_addr,&obj->server_name);    
+    lookuphostname(r_addr,&obj->server_name);
+    if(!(obj->server_ip=(char *)calloc(1,INET_ADDRSTRLEN)))
+      ABORT(R_NO_MEMORY);
+    inet_ntop(AF_INET, r_addr, obj->server_ip, INET_ADDRSTRLEN);
     obj->server_port=r_port;
     
     obj->i_state=SSL_ST_SENT_NOTHING;
@@ -264,7 +274,7 @@ static int create_ssl_analyzer(handle,ctx,conn,objp,i_addr,i_port,r_addr,r_port,
     memcpy(&obj->time_start,base_time,sizeof(struct timeval));
     memcpy(&obj->time_last,base_time,sizeof(struct timeval));    
 
-    if(r=ssl_decoder_create(&obj->decoder,obj->ssl_ctx))
+    if((r=ssl_decoder_create(&obj->decoder,obj->ssl_ctx)))
       ABORT(r);
 
     if (!(obj->extensions=malloc(sizeof(ssl_extensions))))
@@ -273,6 +283,10 @@ static int create_ssl_analyzer(handle,ctx,conn,objp,i_addr,i_port,r_addr,r_port,
     *objp=(proto_obj *)obj;
 
     _status=0;
+  
+    //check logger...
+    if (logger) _status=logger->vtbl->create(&obj->logger_obj,i_addr,i_port,r_addr,r_port,base_time);
+
   abort:
     if(_status){
       destroy_ssl_analyzer((proto_obj **)&obj);
@@ -291,11 +305,16 @@ static int destroy_ssl_analyzer(objp)
     obj=(ssl_obj *)*objp;
     DBG((0,"Destroying SSL analyzer"));
 
+    //check logger...
+    if (logger) logger->vtbl->destroy(&obj->logger_obj);
+
     free_r_queue(obj->i2r_queue);
     free_r_queue(obj->r2i_queue);
     ssl_decoder_destroy(&obj->decoder);
     free(obj->client_name);
+    free(obj->client_ip);
     free(obj->server_name);
+    free(obj->server_ip);
     free(obj->extensions);
     free(*objp);
     *objp=0;
@@ -355,7 +374,7 @@ static int read_ssl_record(obj,q,seg,offset,lastp,offsetp)
 	if (SSL_HEADER_SIZE<q->len)
 	  ABORT(-1);
 	q->read_left=SSL_HEADER_SIZE-q->len;
-	if(r=read_data(q,seg,offset,&last,&offset))
+	if((r=read_data(q,seg,offset,&last,&offset)))
 	  ABORT(r);
         
         q->state=SSL_READ_HEADER;
@@ -387,7 +406,7 @@ static int read_ssl_record(obj,q,seg,offset,lastp,offsetp)
 	q->read_left=rec_len;
 	
       case SSL_READ_HEADER:
-	if(r=read_data(q,last,offset,&last,&offset))
+	if((r=read_data(q,last,offset,&last,&offset)))
 	  ABORT(r);
 	break;
       default:
@@ -437,7 +456,7 @@ static int read_data(q,seg,offset,lastp,offsetp)
     };
     
     if(q->read_left){
-      if(r=copy_tcp_segment_queue(&q->q,seg))
+      if((r=copy_tcp_segment_queue(&q->q,seg)))
 	ABORT(r);
       return(SSL_NO_DATA);
     }
@@ -467,7 +486,7 @@ static int data_ssl_analyzer(_obj,seg,direction)
   {
     int _status,r;
     r_queue *q;
-    segment *last,*q_next,*assembled;
+    segment *last,*q_next=NULL,*assembled;
     ssl_obj *ssl=(ssl_obj *)_obj;
     int offset=0;
     
@@ -514,7 +533,7 @@ static int data_ssl_analyzer(_obj,seg,direction)
 
       ssl->direction=direction;
       
-      if(r=print_ssl_record(ssl,direction,assembled,q->data,q->len))
+      if((r=print_ssl_record(ssl,direction,assembled,q->data,q->len)))
 	ABORT(r);
       
       /*Now reset things, so we can read another record*/
@@ -546,9 +565,7 @@ static int print_ssl_header(obj,direction,q,data,len)
   int len;
   {
     int ct=0;
-    int r;
     segment *s;
-    struct timeval dt;
 
     ssl_print_record_num(obj);
 
@@ -576,8 +593,10 @@ static int print_ssl_record(obj,direction,q,data,len)
   int len;
   {
     int r;
-    
-    if(r=print_ssl_header(obj,direction,q,data,len))
+
+    obj->cur_json_st = json_object_new_object();
+
+    if((r=print_ssl_header(obj,direction,q,data,len)))
       ERETURN(r);
     
     ssl_expand_record(obj,q,direction,data,len);
@@ -586,8 +605,13 @@ static int print_ssl_record(obj,direction,q,data,len)
 
       INIT_DATA(d,data,len);
       exdump(obj,"Packet data",&d);
-      printf("\n\n");
+      LF;LF;
     }
+
+    if(SSL_print_flags & SSL_PRINT_JSON)
+      printf("%s\n", json_object_to_json_string(obj->cur_json_st));
+    json_object_put(obj->cur_json_st);
+    obj->cur_json_st = NULL;
     
     return(0);
   }
@@ -605,12 +629,14 @@ int close_ssl_analyzer(_obj,p,dir)
     else
       what="FIN";
 
+   //check logger...
+    if (logger) logger->vtbl->close(ssl->logger_obj,NULL,0,dir);
+
     explain(ssl,"%d    ",ssl->conn->conn_number);
     ssl_print_timestamp(ssl,&p->ts);
     ssl_print_direction_indicator(ssl,dir);
     explain(ssl,"  TCP %s",what);
-    printf("\n");
-    
+    LF;
     return(0);
   }
 
@@ -620,6 +646,7 @@ static struct proto_mod_vtbl_ ssl_vtbl ={
      parse_ssl_flag,
      create_ssl_ctx,
      create_ssl_analyzer,
+     destroy_ssl_ctx,
      destroy_ssl_analyzer,
      data_ssl_analyzer,
      close_ssl_analyzer,
