@@ -74,14 +74,17 @@ typedef struct ssl_extensions_ {
 
 typedef struct ssl_obj_ {
      tcp_conn *conn;
+     proto_obj *logger_obj;
      int r_state;
      int i_state;
      int version;
-     int cipher_suite;
+     UINT4 cipher_suite;
 
      char *client_name;
+     char *client_ip;
      int client_port;
      char *server_name;
+     char *server_ip;
      int server_port;
 
      struct SSL_CipherSuite_ *cs;
@@ -105,6 +108,7 @@ typedef struct ssl_obj_ {
      int record_count;
      int indent_depth;
      int indent_name_len;
+     struct json_object *cur_json_st;
 } ssl_obj;
 
 typedef struct decoder_ {
